/*
	Document   : main.js
	Created on : 02-Apr-2012, 13:27:45
	Author	 : David Wakelin (david@wakelin.co.cc)
	Description:

jAnimator

Copyright (c) 2012, David Wakelin (Wakelin's)
All rights reserved.
Please see the BSD Licence at the end of this file.


jAnimator is a 'What you see is what you get' (WYSIWYG) animation editor to build jQuery animation code.
It is a web based tool, written in HTML, Javascript, jQuery and CSS that can run from any computer (even without a web server or equivalent software).

Copyright (c) 2012, David Wakelin (Wakelin's)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of the Wakelin's nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/


var getVars = {};
var parts = window.location.href.replace(/[?&]+([^=&]+)=([^&]*)/gi, function(m,key,value) {
	getVars[key] = value;
});


// Define global variables and array's
var cur_ele_id = '';
var that = '';
var ele_id = 0;
var ele_nums = 1;
var recording = false;
var temp = '';

var cur_main_bottom = 45;
var cur_left_bottom = 45;
var cur_bottom_height = 45;
var cur_element_box_height = 45;
var cur_timeline_box_height = 45;

var ele_timeline_order = [];
var ele_status = [];
var ele_html = [];
var ele_height = [];
var ele_width = [];
var ele_top = [];
var ele_left = [];
var ele_index = [];
var ele_opacity = [];
var ele_animations = [];
var ele_animation_details = [];

if (getVars['s']==1) {
	ele_timeline_order = getVars['d'][0];
	ele_status = getVars['d'][1];
	ele_html = getVars['d'][2];
	ele_height = getVars['d'][3];
	ele_width = getVars['d'][4];
	ele_top = getVars['d'][5];
	ele_left = getVars['d'][6];
	ele_index = getVars['d'][7];
	ele_opacity = getVars['d'][8];
	ele_animations = getVars['d'][9];
}

var ani_top = '';
var ani_left = '';
var ani_width = '';
var ani_height = '';

// This function defines all functions for an element and is called each time an element is created.
function define() {

	// Make the element draggable and resizleable on a 10 x 10 grid
	$('.ani_element').draggable({grid: [10, 10]}).resizable({grid: [10, 10]});

	// Set the click for each element in the main panel
	$('.ani_element').mousedown(function(e){
		if (e.button == 2) { // Right click so show context menu
			e.preventDefault();
			$('#context_menu').css({left: e.pageX+'px', top: e.pageY+'px'}).show();
			that = this;
		} else { // Other (left) click so 'select' the element (highlight and infotmation about the current
			 // element in the left panel)
			cur_ele_id = getId(this);
			showDefaults();
		}
	});

	$( ".ani_element" )
	.bind( "dragstop", function(event, ui) { // Update global vars and info with new start position
		if (recording) {
			ani_top = ui.position.top;
			ani_left = ui.position.left;
		} else {
			var id = getId(this);
			cur_ele_id = id;
			ele_left[id] = ui.position.left;
			ele_top[id] = ui.position.top;
			showDefaults();
		}
	})
	.bind( "resizestop", function(event, ui) { // Update global vars and info with new start size
		if (recording) {
			ani_width = Math.round(ui.size.width/10)*10;
			ani_height = Math.round(ui.size.height/10)*10;
		} else {
			var id = getId(this);
			cur_ele_id = id;
			ele_width[id] = Math.round(ui.size.width/10)*10;
			ele_height[id] = Math.round(ui.size.height/10)*10;
			showDefaults();
		}
	});
		
	$('.ele_list_item').mousedown(function(e){ // Set the click for each element in the bottom panel
		/*if (e.button == 2) { // Some problems implementing the context menu for items in this list
			e.preventDefault();
			$('#context_menu').css({left: e.pageX+'px', top: e.pageY+'px'}).show();
			that = this;
		} else {*/// Other (left) click so 'select' the element (highlight and infotmation about the current
			 // element in the left panel)
			cur_ele_id = this.id.substring(14);
			showDefaults();
		//}
	});
}

function getId(that) { // Return the numeric id from an elements ID.
	return (+that.id.replace(/[^0-9]/g, ''));
}

function setDefault(html) { // Set the defaults on all new elements. This function also calls the define and showDefaults functions.
	ele_status[ele_id] = 1;
	ele_html[ele_id] = html;
	ele_height[ele_id] = '100';
	ele_width[ele_id] = '100';
	ele_left[ele_id] = '100';
	ele_top[ele_id] = '100';
	ele_index[ele_id] = 1;
	ele_opacity[ele_id] = 1;
	ele_animations[ele_id] = 'none';
	ele_timeline_order[ele_id] = ele_nums;
	ele_nums = ele_nums + 1;

	expandTimeline(20); // Expand the timeline
		
	$('#main_panel').append(html); // Create the element in the main panel
	$('#element_list_box').append('<div id="ele_list_item_'+ele_id+'" class="ele_list_item">ele_'+ele_id+'</div>'); // Create the timeline item
	$('#ele_list_item_'+ele_id).css('top', (ele_timeline_order[ele_id]*20)+9); // Set the position of the timeline element
		
	define(); // Define the functions for the new element

	cur_ele_id = ele_id;
	showDefaults(); // Update the info in the left panel

	ele_id=ele_id+1; // Increment the element id
}

function showDefaults() { // Update the informaiton in the left panel with the new/selected element
	var id = cur_ele_id;
	$('#cur_id').val(id);
	$('#cur_width').val(ele_width[id]);
	$('#cur_height').val(ele_height[id]);
	$('#cur_left').val(ele_left[id]);
	$('#cur_top').val(ele_top[id]);
	$('#cur_index').val(ele_index[id]);
	$('#cur_opacity').val(ele_opacity[id]);
	$('.ani_element').removeClass('selected')
	$('#ele_'+id).addClass('selected');
	$('.ele_list_item').removeClass('selected');
	$('#ele_list_item_'+id).addClass('selected');
		
}

function expandTimeline(amount) { // Change the height of the timeline panel
	cur_left_bottom = (+$('#left_panel').css('bottom').replace(/[^0-9]/g, ''));
	cur_main_bottom = (+$('#main_panel').css('bottom').replace(/[^0-9]/g, ''));
	cur_bottom_height = (+$('#bottom_panel').css('height').replace(/[^0-9]/g, ''));
	cur_element_box_height = (+$('#element_list_box').css('height').replace(/[^0-9]/g, ''));
	cur_timeline_box_height = (+$('#timeline_box').css('height').replace(/[^0-9]/g, ''));

	$('#left_panel').css('bottom', cur_left_bottom+amount);
	$('#main_panel').css('bottom', cur_main_bottom+amount);
	$('#bottom_panel').css('height', cur_bottom_height+amount);
	$('#element_list_box').css('height', cur_element_box_height+amount);
	$('#timeline_box').css('height', cur_timeline_box_height+amount);
}

function moveTimeline(left) {
	$('#time_bar').css({ left: left + 'px' });
	temp = left;
	$.each(ele_animation_details, function(i, v) {
		if (v.time/10 > temp) {
			$('#ele_'+v.parent).css({
				top: v.orig_top + "px",
				left: v.orig_left + "px",
				height: v.orig_height + "px",
				width: v.orig_width + "px"
			});
		} else if ((v.time+v.duration)/10 < temp) {
			$('#ele_'+v.parent).css({
				top: v.top + "px",
				left: v.left + "px",
				height: v.height + "px",
				width: v.width + "px"
			});
		} else {
			var percentage_offset = ((temp - (v.time / 10)) / (v.duration / 10)) * 100;
			var new_top = Math.round(((((v.top - v.orig_top) * percentage_offset) / 100) + v.orig_top)/10)*10;
			var new_left = Math.round(((((v.left - v.orig_left) * percentage_offset) / 100) + v.orig_left)/10)*10;
			var new_height = Math.round(((((v.height - v.orig_height) * percentage_offset) / 100) + v.orig_height)/10)*10;
			var new_width = Math.round(((((v.width - v.orig_width) * percentage_offset) / 100) + v.orig_width)/10)*10;
			$('#ele_'+v.parent).css({
				
				top: new_top + "px",
				left: new_left + "px",
				height: new_height + "px",
				width: new_width + "px"
			});
		}
	});
}

$(document).ready(function(){ // All run define once functions for statcic elements

	window.oncontextmenu = function () { // Prevent standard context menu
	return false;
	}

	$(document).mouseup(function(e){ // Hide the context menu on non-right click
		if (e.button != 2) {
			$('#context_menu').hide();
		}
	});
	
	// Make the time bar draggable
	$('#time_bar').draggable({grid: [10, 10], axis: "x", containment: "#timeline_box", scroll: false})
	.bind( "dragstop", function(event, ui) {
		moveTimeline(ui.position.left);
	});

	$('#add_div').click(function(){ // Add new div element
		setDefault('<div class="ani_element" id="ele_'+ele_id+'"></div>');
	});

	$('#add_img').click(function(){ // Add new img element
		setDefault('<div class="ani_element" id="ele_'+ele_id+'"><img src=""/>Image</div>');
	});

	$('#cur_left_go').click(function(){ // Set the left start value
		ele_left[cur_ele_id] = $('#cur_left').val();
		$('#ele_'+$('#cur_id').val()).css({left: $('#cur_left').val()+'px'});
	});

	$('#cur_top_go').click(function(){ // Set the top start value
		ele_top[cur_ele_id] = $('#cur_top').val();
		$('#ele_'+$('#cur_id').val()).css({top: $('#cur_top').val()+'px'});
	});

	$('#cur_width_go').click(function(){ // Set the width start value
		ele_width[cur_ele_id] = $('#cur_width').val();
		$('#ele_'+$('#cur_id').val()).css({width: $('#cur_width').val()+'px'});
	});

	$('#cur_height_go').click(function(){ // Set the height start value
		ele_height[cur_ele_id] = $('#cur_height').val();
		$('#ele_'+$('#cur_id').val()).css({height: $('#cur_height').val()+'px'});
	});

	$('#cur_index_go').click(function(){ // Set the z-index start value
		ele_index[cur_ele_id] = $('#cur_index').val();
		$('#ele_'+$('#cur_id').val()).css({'z-index': $('#cur_index').val()});
	});

	$('#cur_opacity_go').click(function(){ // Set the opacity start value
		ele_opacity[cur_ele_id] = $('#cur_opacity').val();
		$('#ele_'+$('#cur_id').val()).css({opacity: $('#cur_opacity').val()});
	});
	
	$('#add_animation').click(function(){
		cur_ele_id = getId(that);
		showDefaults();
		recording = true;
		$('#recording_animation').show();
		var pos = $(that).position();
		ani_top = pos.top;
		ani_left = pos.left;
		ani_height = $(that).height();
		ani_width = $(that).width();
		temp = ani_top+'_'+ani_left+'_'+ani_height+'_'+ani_width;
	});
	
	$('#recording_animation').click(function(){
		$('#recording_animation').hide();
		recording = false;
		var animation = new Object();
		animation.parent = cur_ele_id;
		animation.time = 0;
		animation.duration = 600;
		animation.top = ani_top;
		animation.left = ani_left;
		animation.width = ani_width;
		animation.height = ani_height;
		var vals = temp.split('_');
		animation.orig_top = Number(vals[0]);
		animation.orig_left = Number(vals[1]);
		animation.orig_width = Number(vals[2]);
		animation.orig_height = Number(vals[3]);
		
		ele_animation_details.push(animation);
		var last_id = ele_animation_details.length-1
		if (ele_animations[cur_ele_id] == 'none') {
			ele_animations[cur_ele_id] = last_id;
		} else {
			ele_animations[cur_ele_id] = ele_animations[cur_ele_id] + ',' + last_id;
		}
		$('#ele_'+cur_ele_id).css({top: ele_top[cur_ele_id]+'px'});
		$('#ele_'+cur_ele_id).css({left: ele_left[cur_ele_id]+'px'});
		$('#ele_'+cur_ele_id).css({width: ele_width[cur_ele_id]+'px'});
		$('#ele_'+cur_ele_id).css({height: ele_height[cur_ele_id]+'px'});
		
		$('#timeline_box').append('<div id="ele_timeline_item_'+last_id+'" class="ele_timeline_item">1</div>'); // Create the timeline item
		$('#ele_timeline_item_'+last_id).css('top', (ele_timeline_order[cur_ele_id]*20)+9); // Set the position of the timeline element
		$('#ele_timeline_item_'+last_id).css('left', '0px'); // Set the position of the timeline element
		$('#ele_timeline_item_'+last_id).css('width', '60px'); // Set the position of the timeline element
		$('#ele_timeline_item_'+last_id).css('height', '20px'); // Set the position of the timeline element
		$('#ele_timeline_item_'+last_id).draggable({grid: [10, 10], axis: "x"}).resizable({grid: [10, 10], maxHeight: 20, minHeight: 20});
		
		$('.ele_timeline_item')
		.bind( "dragstop", function(event, ui) { // Update global vars and info with new start time
			var ani_id = getId(this);
			ele_animation_details[ani_id].time = ui.position.left*10;
		})
		.bind( "resizestop", function(event, ui) { // Update global vars and info with new duration
			var ani_id = getId(this);
			ele_animation_details[ani_id].duration = Math.round(ui.size.width/10)*100;
		});
		
	});

	$('#remove_ele').click(function(){ // Remove an element
		cur_ele_id = getId(that);
		showDefaults();
		ele_status[cur_ele_id] = 0;
		var ani_ids = '';
		$('#ele_list_item_'+cur_ele_id).remove();
		$(that).draggable( "destroy" ).resizable( "destroy" ).remove();
		$('.ele_list_item').each(function(){ // Decrease the timeline index for each element after this one
			if (ele_timeline_order[this.id.substring(14)] > ele_timeline_order[cur_ele_id]) {
				var cur_ele_list_item_top = (+$(this).css('top').replace(/[^0-9]/g, ''));
				$(this).css('top', cur_ele_list_item_top-20);
				ele_timeline_order[this.id.substring(14)] = ele_timeline_order[this.id.substring(14)] - 1;
				ani_ids = new String(ele_animations[this.id.substring(14)]).split(',');
				$.each(ani_ids, function(i, v) {
					$('#ele_timeline_item_'+v).css('top', cur_ele_list_item_top-20);
				});
			}
		});
		ani_ids = new String(ele_animations[cur_ele_id]).split(',');
		
		$.each(ani_ids, function(i, v) {
			$('#ele_timeline_item_'+v).remove();
		});
		expandTimeline(-20);
		ele_nums = ele_nums - 1; // Decrease the number of elements in the timeline panel
	});
	
	$('#preview_all').click(function(){
		moveTimeline(0);
		var time, preview_end = Number(-1), animation_end = Number(0);
		$.each(ele_animation_details, function(i, v) {
			if (v.time > preview_end) {
				preview_end = v.time; // Get time of last animation
				animation_end = v.time+v.duration+1000;
			}
		});
		$('#time_bar').animate({ left: animation_end/10+'px' }, animation_end, 'linear');
		setTimeout(
			function() {
				$('#time_bar').css({ left: '0px' });
			},
			animation_end+100
		);
		for (time=0;time<=preview_end;time=time+100) {
			$.each(ele_animation_details, function(i, v) {
				if (v.time==time) {
					setTimeout(
						function() { 
							$('#ele_'+v.parent).animate({
								top: v.top+'px',
								left: v.left+'px',
								height: v.height+'px',
								width: v.width+'px'
							}, v.duration);
						},
						v.time
					);
				}
			});
		}
		$.each(ele_status, function(i, v) {
			setTimeout(
				function() { 
					$('#ele_'+i).css({
						top: ele_top[i]+'px',
						left: ele_left[i]+'px',
						height: ele_height[i]+'px',
						width: ele_width[i]+'px'
					});
				},
				animation_end
			);
		});
	});
	
	$('#save').click(function(){
		var output = [];
		output.push(ele_timeline_order);
		output.push(ele_status);
		output.push(ele_html);
		output.push(ele_height);
		output.push(ele_width);
		output.push(ele_top);
		output.push(ele_left);
		output.push(ele_index);
		output.push(ele_opacity);
		output.push(ele_animations);
		output.push(ele_animation_details);
		alert(JSON.stringify(output));
	});
});